from roboflowoak.pipe import DepthAIPipeline, list_devices
from roboflowoak.api import download_blob
import os
import json
import threading
from pip._internal.utils.appdirs import user_cache_dir
import shutil

cache_dir = user_cache_dir('pip') + "/roboflow"

class RoboflowOak:
    def __init__(self, model, version, api_key, confidence=0.2, overlap=0.5, rgb=False, depth=False, device=None, device_name="device", blocking=True, legacy=False):
        self.project = model
        self.version = version
        self.api_key = api_key
        self.blocking = blocking
        self.depth = depth
        self.confidence = confidence
        self.overlap = overlap
        self.device_name = device_name
        self.dev = device

        if self.dev is None:
            available_devices = list_devices()
            if len(available_devices) == 0:
                raise Exception("No available OAK devices detected.")
                default_device = None
            else:
                self.dev = available_devices[0]

        self.cache_path = self.find_weights()
        with open(os.path.join(self.cache_path, 'config.txt'), 'r') as f:
            self.model_objects = json.loads(f.read())
            self.size = (int(self.model_objects["environment"]["RESOLUTION"]), int(self.model_objects["environment"]["RESOLUTION"]))
            self.resolution = self.size
            self.class_names = self.model_objects["class_names"]
            self.colors = self.model_objects["colors"]
        try:
            self.dai_pipe = DepthAIPipeline(self.cache_path+"/roboflow.blob", self.size, self.resolution, self.class_names, rgb, self.colors, self.confidence, self.overlap, False, depth, device, legacy)
        except:
            try:
                shutil.rmtree(cache_dir)
                self.cache_path = self.find_weights()
                with open(os.path.join(self.cache_path, 'config.txt'), 'r') as f:
                    self.model_objects = json.loads(f.read())
                    self.size = (int(self.model_objects["environment"]["RESOLUTION"]), int(self.model_objects["environment"]["RESOLUTION"]))
                    self.resolution = self.size
                    self.class_names = self.model_objects["class_names"]
                    self.colors = self.model_objects["colors"]
                self.dai_pipe = DepthAIPipeline(self.cache_path+"/roboflow.blob", self.size, self.resolution, self.class_names, rgb, self.colors, self.confidence, self.overlap, False, depth, device, legacy)
            except:
                raise Exception("Failure while retrying load weights - does this model, version, api key exist? can you curl api.roboflow.com, and can your device download files from google cloud storage? have you hit your device limit?")

    def find_weights(self):
        if os.path.exists(os.path.join(cache_dir, self.project, self.version, "roboflow.blob")) and os.path.exists(
                        os.path.join(cache_dir, self.project, self.version, "config.txt")):
            return os.path.join(cache_dir, self.project, self.version)

        return download_blob(self.project, self.version, self.api_key, self.dev)

    def detect(self, visualize=True):
        if self.blocking:
            ret = self.dai_pipe.get()         
        else:
            ret = self.dai_pipe.try_get()

        predictions = []
        dets = ret[0]
        for det in dets:
            if len(det) > 6:
                predictions.append(Prediction((det[0]+det[2])/2, (det[1]+det[3])/2, det[2]-det[0], det[3]-det[1], det[5], det[4], det[6]))
            else:
                predictions.append(Prediction((det[0]+det[2])/2, (det[1]+det[3])/2, det[2]-det[0], det[3]-det[1], det[5], det[4]))

        ret = list(ret)
        ret[0] = {"predictions": predictions}
        return ret

class OAKThread(threading.Thread, RoboflowOak):
    def __init__(self, threadID, name, counter, onDetect, project, version, api_key, confidence=0.5, overlap=0.5, rgb=True, depth=False, device=None, device_name="device", blocking=True, legacy=False):
        OAK.__init__(self, project, version, api_key, confidence, overlap, rgb, depth, device, device_name, blocking, legacy)
        threading.Thread.__init__(self)
        self.threadID = threadID
        self.name = name
        self.counter = counter
        self.onDetect = onDetect

    def run(self):
        while True:
            detections, frame, raw_frame, depth = self.detections()
            self.onDetect(self.threadID, detections, frame, raw_frame, depth)



def devices():
    return list_devices()

class Prediction:
    def __init__(self, x, y, width, height, confidence, class_name, depth=None):
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.depth = depth
        self.confidence = confidence
        self.class_name = class_name

    def json(self):
        return {
            "x": self.x,
            "y": self.y,
            "width": self.width,
            "height": self.height,
            "depth": self.depth,
            "confidence": self.confidence,
            "class": self.class_name
        }


import requests
import os
import json
from pip._internal.utils.appdirs import user_cache_dir

api_endpoint = "https://api.roboflow.com/oak/"
cache_dir = user_cache_dir('pip') + "/roboflow"

def download_blob(project, version, api_key, device_id):

    endpoint = api_endpoint+project+"/"+version+"?api_key="+api_key+"&device="+device_id
    api_data = requests.get(endpoint)

    if api_data.status_code != 200:
        raise Exception(str(api_data.json()))
    api_data = api_data.json()
    if 'warning' in api_data.keys():
        print(api_data['warning'], flush=True)

    model_objects = {}
    model_objects['class_names'] = api_data['oak']['classes']
    model_objects['colors'] = api_data['oak']['colors']
    model_objects['environment'] = requests.get(api_data['oak']['environment']).json()
    r = requests.get(api_data['oak']['model'])
    try:
        os.makedirs(os.path.join(cache_dir, project, version))
    except OSError as error:
        print(error)
    with open(os.path.join(cache_dir, project, version, 'roboflow.blob'), 'wb') as f:
        f.write(r.content)
    with open(os.path.join(cache_dir, project, version, 'config.txt'), 'w') as f:
        f.write(json.dumps(model_objects))

    return os.path.join(cache_dir, project, version)



import depthai as dai
import numpy as np
import cv2
import roboflowoak.postprocs as postprocs


class DepthAIPipeline:
    def __init__(self, nn_path, size, resolution, class_names, cam_stream, colors, confidence=0.5, overlap=0.5, stretch=False, depth=False, device=None, legacy=False):
        self.nn_path = nn_path
        self.size = size
        self.class_names = class_names
        self.cam_stream = cam_stream
        self.colors = colors
        self.stretch = stretch
        self.resolution = resolution
        self.dev = device
        self.depth = depth
        self.overlap = overlap
        self.confidence = confidence

        self.pipeline = dai.Pipeline()
        if legacy:
            self.pipeline.setOpenVINOVersion(dai.OpenVINO.Version.VERSION_2021_1)

        self.cam_rgb = self.pipeline.createColorCamera()
        self.cam_rgb.setPreviewSize(self.resolution)
        self.cam_rgb.setInterleaved(False)

        self.detection_nn = self.pipeline.createNeuralNetwork()
        try:
            self.detection_nn.setBlobPath(self.nn_path)
        except:
            raise Exception("Failure loading model...")


        if cam_stream:
            self.xout_rgb = self.pipeline.createXLinkOut()
            self.xout_rgb.setStreamName("rgb")
            self.cam_rgb.preview.link(self.xout_rgb.input)

        self.xout_nn = self.pipeline.createXLinkOut()
        self.xout_nn.setStreamName("nn")
        self.detection_nn.out.link(self.xout_nn.input)

        if self.stretch:
            self.manip = self.pipeline.createImageManip()

            self.manip.initialConfig.setResize(self.size)
            self.manip.inputImage.setBlocking(True)

            self.manip.out.link(self.detection_nn.input)
            self.cam_rgb.preview.link(self.manip.inputImage)
        else:
            self.cam_rgb.preview.link(self.detection_nn.input)

        if self.depth:
            self.left = self.pipeline.create(dai.node.MonoCamera)
            self.right = self.pipeline.create(dai.node.MonoCamera)
            self.stereo = self.pipeline.create(dai.node.StereoDepth)

            self.depthOut = self.pipeline.create(dai.node.XLinkOut)

            self.depthOut.setStreamName("depth")

            monoResolution = dai.MonoCameraProperties.SensorResolution.THE_400_P
            fps = 30

            self.left.setResolution(monoResolution)
            self.left.setBoardSocket(dai.CameraBoardSocket.LEFT)
            self.right.setResolution(monoResolution)
            self.right.setBoardSocket(dai.CameraBoardSocket.RIGHT)

            self.stereo.initialConfig.setConfidenceThreshold(245)
            self.stereo.setLeftRightCheck(True)
            self.stereo.setDepthAlign(dai.CameraBoardSocket.RGB)

            self.left.out.link(self.stereo.left)
            self.right.out.link(self.stereo.right)
            self.stereo.disparity.link(self.depthOut.input)


        if self.dev is None:
            available_devices = list_devices()
            if len(available_devices) == 0:
                default_device = None
            else:
                self.dev = available_devices[0]

        found, device_info = dai.Device.getDeviceByMxId(self.dev)

        if not found:
            print("Device Not Found")\

        self.device = dai.Device(self.pipeline, device_info)

        self.q_det = self.device.getOutputQueue(name="nn", maxSize=4, blocking=False)

        if self.depth:
            self.q_depth = self.device.getOutputQueue(name="depth", maxSize=4, blocking=False)

        if cam_stream:
            self.q_rgb = self.device.getOutputQueue(name="rgb", maxSize=4, blocking=False)

    def disparity_to_depth(self, disparity):
        return 441.25 * 7.5 / disparity

    def detection_depth(self, detections, depth):
        res = []
        sx = len(depth)/self.size[0]
        sy = len(depth[0])/self.size[1]

        depth = np.nan_to_num(depth, copy=False, nan=0)

        depth_map = self.disparity_to_depth(depth)

        for det in detections:
            x = (det[2]+det[0])/2
            y = (det[3]+det[1])/2
            x = int(x*sx)
            y = int(y*sy)
            d = depth_map[det[0]:det[2], det[1]:det[3]]
            dist = np.amin(d)
            res.append([det[0], det[1], det[2], det[3], det[4], det[5], dist])

        return res

    def get(self):
        in_det = self.q_det.get()

        detections = self.post_processing(in_det)

        depth = None

        if self.depth:
            in_depth = self.q_depth.get()
            depth = in_depth.getFrame()
            detections = self.detection_depth(detections, depth)


        if self.cam_stream:
            in_rgb = self.q_rgb.get()
            frame, frame_raw = self.process_frame(detections, in_rgb)
            return detections, frame, frame_raw, depth

        return detections, depth

    def try_get(self):
        in_det = self.q_det.tryGet()

        detections = self.post_processing(in_det)

        depth = None

        if self.depth:
            in_depth = self.q_depth.get()
            depth = in_depth.getFrame()
            detections = self.detection_depth(detections, depth)

        if self.cam_stream:
            in_rgb = self.q_rgb.tryGet()
            frame, frame_raw = self.process_frame(detections, in_rgb)
            return detections, frame, frame_raw, depth

        return detections, depth

    def process_frame(self, detections, in_rgb):
        frame_raw = None
        frame = None
        if in_rgb is not None:
            shape = (3, in_rgb.getHeight(), in_rgb.getWidth())
            frame = in_rgb.getData()
            frame = frame.reshape(shape)
            frame = frame.transpose(1, 2, 0)
            frame = frame.astype(np.uint8)
            frame = np.ascontiguousarray(frame)
            frame_raw = np.array(frame)


        if frame is not None:
            for detection in self.scale_detections(detections):
                class_color = self.colors[detection[4]].strip("#")
                class_color = tuple(int(class_color[i:i + 2], 16) for i in (0, 2, 4))
                cv2.rectangle(frame, (detection[0], detection[1]), (detection[2], detection[3]), class_color, 2)
                cv2.putText(frame, detection[4], (detection[0] + 10, detection[1] + 20), cv2.FONT_HERSHEY_TRIPLEX, 0.5,
                            class_color)
                cv2.putText(frame, str(int(detection[5] * 100)) + "%", (detection[0] + 10, detection[1] + 40),
                            cv2.FONT_HERSHEY_TRIPLEX, 0.5, class_color)

        return frame, frame_raw

    def scale_detections(self, detections):
        sx = self.resolution[0]/self.size[0]
        sy = self.resolution[1]/self.size[1]
        res = []
        for det in detections:
            res.append([
                int(det[0]*sx), int(det[1]*sy), int(det[2]*sx), int(det[3]*sy), det[4], det[5]
            ])

        return res

    def post_processing(self, in_det):
        if in_det == None:
            return []

        in_nn_layer = in_det.getLayerFp16('output')
        num_anchor_boxes = len(np.array(in_nn_layer)) / (len(self.class_names) + 5)
        tensors = np.reshape(np.array(in_nn_layer), (1, int(num_anchor_boxes), len(self.class_names) + 5))

        batch_detections_np = postprocs.w_np_non_max_suppression(tensors, len(self.class_names), conf_thres=self.confidence, nms_thres=self.overlap)

        detections = []
        if len(batch_detections_np) == 0:
            detections = []
        else:
            detections = postprocs.process_detections(batch_detections_np[0], self.size, class_filter=self.class_names,
                                            class_names=self.class_names)

        return detections

def list_devices():
    available_devices = []
    for device in dai.Device.getAllAvailableDevices():
        available_devices.append(device.getMxId())
    return available_devices





  
import setuptools
from setuptools import find_packages

with open("README-dist.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="roboflowoak",  # Replace with your own username
    version="0.0.4",
    author="Roboflow",
    license='GPLv3+',
    author_email="jacob@roboflow.com",
    description="python client for deploying Roboflow models to OAK devices",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.roboflow.com",
    install_requires=[
        "requests==2.26.0"
    ],
    packages=find_packages(include=('roboflowoak',)),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)